<?php

namespace App\Http\Controllers;

use App\Http\Requests\DraftOrderValidation;
use Illuminate\Http\Request;
use App\Model\Shopify\Product;
use App\Model\CustomOrderModel;
// use OhMyBrew\BasicShopifyAPI;
use App\Http\ResponseHelper;
use JsonSerializable;
// use OhMyBrew\ShopifyApp\Facades\ShopifyApp;
use Session;

use Illuminate\Support\Facades\Auth;

use Config;

class OrderController extends Controller
{

    private $productObj;
    private $helper;
    private $shop;
    private $domain;
    private $subscriptionProductId;
    private $subscriptionVariantId;
    //private $orderModelObj;


    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {

        $this->productObj = new Product();
        $this->helper = new ResponseHelper();
        //$this->orderModelObj = new CustomOrderModel();

        $this->subscriptionProductId = env('SUBSCRIPTION_PRODUCT_ID', true);
        $this->subscriptionVariantId = env('SUBSCRIPTION_VARIANT_ID', true);

        if (isset($_REQUEST['shop'])) {
            $this->domain = $_REQUEST['shop'];
        } else {
            $this->domain = env('SHOP_DOMAIN', true);
        }
    }

    // To create order view
    public function createOrder($status = [])
    {

        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod);
        $status = json_encode($status);

        return view('order', compact('jsonProductDetails'), compact('status'));
    }

    // To store the order in shopify
    public function saveOrder(DraftOrderValidation $req)
    {

        $customerResponse = '';
        $setOrderRequest = $this->setOrderRequest($req);
        $setCustomerRequest = $this->setCustomerRequest($req);
        $shop = Auth::user();

        $customOrderDetails = [];
        $params['order'] = $setOrderRequest;

        $result = $shop->api()->rest('POST', htmlentities('/admin/orders.json'), $params);
        $result = (object)$result;


        $customerUpdateDetails = [];

        if (!empty($result->body->order->customer->id)) {

            $customerUpdateDetails['id'] = $result->body->order->customer->id;
            $customerUpdateDetails['first_name'] = !empty($req->firstName)
                ? $req->firstName : '';
            $customerUpdateDetails['last_name'] = !empty($req->lastName)
                ? $req->lastName : '';

            $updateParam['customer'] = $customerUpdateDetails;

            $customerResponse = $shop->api()->rest(
                'PUT',
                htmlentities('/admin/customers/' . $result->body->order->customer->id . '.json'),
                $updateParam
            );
        }

        //  if (!empty($result->body->order->id)) {

        //     $customOrderDetails = [
        //         'order_id' => $result->body->order->name,
        //         'email' => $result->body->order->email,
        //         'shopify_order_id' => $result->body->order->id
        //     ];

        //     $this->orderModelObj->insertCustomOrderDetails($customOrderDetails);
        //  }

        $prod  = $this->setProductDetails();
        $jsonProductDetails = json_encode($prod);
        return redirect()->action('OrderController@createOrder', ['status' => 'success']);
    }

    public function setCustomerRequest($reqObj)
    {

        $customerRequest['customer']['first_name'] = $reqObj->firstName;
        $customerRequest['customer']['last_name']  = $reqObj->lastName;
        $customerRequest['customer']['email']      = $reqObj->email;
        // $customerRequest['customer']['addresses']  = array(
        //     array(
        //         "address1" => $reqObj->shipAddress1,
        //         "city"=> $reqObj->shipCity,
        //         "province"=> $reqObj->shipProvince,
        //         "zip"=> $reqObj->shipZip,
        //         "last_name"=> $reqObj->shipLastname,
        //         "first_name"=> $reqObj->shipFirstname,
        //         "country"=> $reqObj->shipCountry,
        //     )
        // );
        return $customerRequest;
    }

    // To set the request for order and draft order
    public function setOrderRequest($reqObj)
    {
        $validated = $reqObj->validated();
        $request = [];
        $lineProperTies = [];
        $prod  = $this->setProductDetails();
        $i = 1;
        $price = 0;
        $unit_price = '';

        $packDetails = [];
        foreach ($reqObj['prod-id'] as $key => $productId) {
            $propertyValue = '';

            if ($i < 10) {
                $propertyKey = 'Vitamin0' . $i++;
            } else if ($i >= 10) {
                $propertyKey = 'Vitamin' . $i++;
            }
            if (isset($prod['products'][$productId])) {

                if (isset($reqObj['prod-variant-id'][$key])) {
                    $variantId = $reqObj['prod-variant-id'][$key];
                    $sku = $prod["products"][$productId]['variants'][$variantId]['sku'];

                    $temp = [];
                    $temp['bulkID'] = $sku;
                    $temp['qty'] = $reqObj['prod-qty'][$key];
                    array_push($packDetails, $temp);

                    if (isset($reqObj['prod-qty'][$key])) {

                        $propertyValue .= $prod['products'][$productId]['title'] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['title'] . ' | ' .
                            $reqObj['prod-qty'][$key] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['id'] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['sku'];

                        $lineProperTies[] = ['name' => $propertyKey, 'value' => $propertyValue];
                        $unit_price = ($prod["products"][$productId]['variants'][$variantId]['price']) * $reqObj['prod-qty'][$key];
                        $price += $unit_price;
                    }
                }
            }
        }

        $packData = $this->getPackCount($packDetails);

        if (!empty($packData)) {

            $lineProperTies[] = ['name' => 'Packs', 'value' => $packData];
        }

        // $nextOrderId = '';
        // $lastCustomOrderId = $this->orderModelObj->getCustomOrderDetails();
        // if(!empty($lastCustomOrderId)) {

        //     $customOrderId = explode('-',$lastCustomOrderId);
        //     $orderNumber = (int)$customOrderId[1];
        //     $incrementId = $orderNumber + 1;
        //     $nextOrderId = $customOrderId[0].'-'.$incrementId;

        // }
        // else{
        //     $nextOrderId = "#custom-1";
        // }

        $request['customer']['first_name'] = $reqObj->firstName;
        $request['customer']['last_name'] = $reqObj->lastName;
        $request['customer']['email'] = $reqObj->email;
        //$request['name'] = $nextOrderId;
        $request['email'] = $reqObj->email;
        $request['tags'] = "custom-order-tool-created,".$reqObj->shOrderTags;

        // echo $this->subscriptionProductId;
        // echo $this->subscriptionVariantId;die;
        $request['line_items'] = array(
            array(
                'title' => "Your Monthly Subscription",
                'product_id' => $this->subscriptionProductId,
                'variant_id' => $this->subscriptionVariantId,
                'price' => $price,
                'quantity' => '1',
                'properties' => $lineProperTies
            )
        );

        if (empty($reqObj->shipAddress1)) {
            $customerDetails = $this->getCustomerAddress($reqObj->email);

            if (!empty($customerDetails)) {
                $custDetails = $customerDetails->body->customers[0];
                $customerId = $custDetails->id;
                $address1 = !empty($custDetails->default_address->address1) ? $custDetails->default_address->address1 : null;
                $address2 = !empty($custDetails->default_address->address2) ? $custDetails->default_address->address2 : null;
                $city = !empty($custDetails->default_address->city) ? $custDetails->default_address->city : null;
                $country = !empty($custDetails->default_address->country) ? $custDetails->default_address->country : null;
                $firstName = !empty($custDetails->default_address->first_name) ? $custDetails->default_address->first_name : null;
                $lastName = !empty($custDetails->default_address->last_name) ? $custDetails->default_address->last_name : null;
                $province = !empty($custDetails->default_address->province_code) ? $custDetails->default_address->province_code : null;
                $zip = !empty($custDetails->default_address->zip) ? $custDetails->default_address->zip : null;

                $request['shipping_address'] = array(
                    "address1" => $address1,
                    "address2" => $address2,
                    "city" => $city,
                    "country" => $country,
                    "first_name" => $firstName,
                    "last_name" => $lastName,
                    "province" => $province,
                    "zip" => $zip,
                );
            } else {
                $request['billing_address'] = null;
            }
        } else {
            $request['shipping_address'] = array(
                "address1" => $reqObj->shipAddress1,
                "address2" => $reqObj->shipAddress2,
                "city" => $reqObj->shipCity,
                "country" => $reqObj->shipCountry,
                "first_name" => $reqObj->shipFirstname,
                "last_name" => $reqObj->shipLastname,
                "province" => $reqObj->shipProvince,
                "zip" => $reqObj->shipZip,
            );
        }
        if (empty($reqObj->billAddress1)) {

            $customerDetails = $this->getCustomerAddress($reqObj->email);

            if (!empty($customerDetails)) {
                $custDetails = $customerDetails->body->customers[0];
                $customerId = $custDetails->id;
                $address1 = !empty($custDetails->default_address->address1) ? $custDetails->default_address->address1 : null;
                $address2 = !empty($custDetails->default_address->address2) ? $custDetails->default_address->address2 : null;
                $city = !empty($custDetails->default_address->city) ? $custDetails->default_address->city : null;
                $country = !empty($custDetails->default_address->country) ? $custDetails->default_address->country : null;
                $firstName = !empty($custDetails->default_address->first_name) ? $custDetails->default_address->first_name : null;
                $lastName = !empty($custDetails->default_address->last_name) ? $custDetails->default_address->last_name : null;
                $province = !empty($custDetails->default_address->province_code) ? $custDetails->default_address->province_code : null;
                $zip = !empty($custDetails->default_address->zip) ? $custDetails->default_address->zip : null;

                $request['billing_address'] = array(
                    "address1" => $address1,
                    "address2" => $address2,
                    "city" => $city,
                    "country" => $country,
                    "first_name" => $firstName,
                    "last_name" =>  $lastName,
                    "province" => $province,
                    "zip" =>  $zip,
                );
            } else {

                $request['billing_address'] = null;
            }
        } else {
            $request['billing_address'] = array(
                "address1" => $reqObj->billAddress1,
                "address2" => $reqObj->billAddress2,
                "city" => $reqObj->billCity,
                "country" => $reqObj->billCountry,
                "first_name" => $reqObj->billFirstname,
                "last_name" => $reqObj->billLastname,
                "province" => $reqObj->billState,
                "zip" => $reqObj->billZip,
            );
        }

        //$request['tags'] = $reqObj->shOrderTags;

        //echo"<pre>"; print_r($request);die('test');
        return $request;
    }

    public function getCustomerAddress($email)
    {

        $customerObj = Auth::user();
        $customerEmail = $email;
        $params = ["query" => "email:{$customerEmail}"];
        $result = $customerObj->api()->rest('GET', htmlentities('/admin/customers/search.json'), $params);

        if (isset($result->body->customers) && empty($result->body->customers)) {
            return null;
        }

        return $result;
    }

    public function getCustomerAddress2(Request $req)
    {
        $shop = Auth::user();
        $limit = 250;
        $customerEmail = $req->email;
        $params = ["limit" => $limit, "query" => "email:{$customerEmail}"];
        $result = $shop->api()->rest('GET', htmlentities('/admin/customers/search.json'), $params);

        $result = (object)$result;

        if (isset($result->body->customers) && empty($result->body->customers)) {
            return null;
        }


        foreach ($result->body->customers as $customer) {

            // echo "<pre>".$customer['email'].'-----'.PHP_EOL;
            if (trim($customer['email']) == trim($customerEmail)) {

                $customerDetails['body']['customers'][0] = $customer;
                return json_encode($customerDetails);
            }
        }

        return null;
    }

    // To set the shopify product details
    private function setProductDetails()
    {

        $vari = [];
        $prod['products'] = [];
        $prod  = [];
        $limit = 250;
        $nextLink = '';
        $previousLink = '';
        $prodCount = 0;

        $shop    = Auth::user();

        do {

            try {

                if (empty($previousLink) && empty($nextLink)) {

                    $productDetails = $shop->api()->rest(
                        'GET',
                        '/admin/products.json',
                        ["published_status" => "published", "fields" =>
                        "id,title,variants", "limit" => $limit]
                    );

                    $productDetails = (object)$productDetails;

                    $previousLink = !empty($productDetails->link->previous) ?
                        $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                        $productDetails->link->next : '';
                } elseif (!empty($nextLink)) {

                    $params = [
                        "fields" => "id,title,variants", "limit" => $limit,
                        'page_info' => $nextLink
                    ];
                    $productDetails = $shop->api()->rest('GET', '/admin/products.json', $params);
                    $productDetails = (object)$productDetails;

                    $previousLink = !empty($productDetails->link->previous) ?
                        $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                        $productDetails->link->next : '';
                }
            } catch (Exception $e) {
                echo PHP_EOL . "Error when fetching the products from shopify and the error is -  "
                    . $e->getMessage();
                die;
            }


            $productDetails = $productDetails->body;

            if (!empty($productDetails)) {

                $subscription_product = array('1809168695367', '4359186350162');

                foreach ($productDetails->products as $products) {
                    if (in_array($products->id, $subscription_product)) {
                        continue;
                    }


                    foreach ($products->variants as $variants) {
                        $prodCount++;

                        $vari[$variants->id] = [
                            'id' => $variants->id,
                            'product_id' => $variants->product_id,
                            'title' => strip_tags($variants->title),
                            'sku' => $variants->sku,
                            'price' => $variants->price
                        ];
                    }
                    $prod['products'][$products->id] = [
                        'id' => $products->id,
                        'title' => strip_tags($products->title), 'variants' => $vari
                    ];
                    $vari = [];
                }
            }
            $count = count($productDetails->products);
            if (empty($nextLink)) {
                break;
            }
        } while ($limit == $count);

        return $prod;
    }


    public function getPackCount($request)
    {

        $postData = $request;

        if (empty($postData)) {
            $returnData = [
                'error' => 'Empty parameters'
            ];

            return $this->helper->returnData(600, "error", $returnData);
        }

        $packRequest = array();

        foreach ($postData as $data) {
            $bulkID = isset($data['bulkID']) ? $data['bulkID'] : '';
            $qty = isset($data['qty']) ? $data['qty'] : '';

            if (empty($bulkID)) {
                $returnData = [
                    'error' => 'Empty parameter value - bulkIDs'
                ];

                return $returnData;
                break;
            }

            if (empty($qty)) {
                $returnData = [
                    'error' => 'Empty parameter value - qty'
                ];

                return $returnData;
            }

            if ($bulkID == 'vitamin-pack') {
                continue;
            }

            // $tempPackRequest['BulkItem'] = $bulkID;
            $tempPackRequest['bulkID'] = $bulkID;
            // $tempPackRequest['Quantity'] = $qty;
            $tempPackRequest['qty'] = $qty;
            array_push($packRequest, $tempPackRequest);
        }

        // if(empty($packRequest['ProductItems'])) {

        //     $returnData = [
        //         'error' => 'Empty ProductItems'
        //     ];

        //     return  $returnData ;
        // }
        $returnASCII = $this->encode_items($packRequest);

        $packDetails = $this->callGetPack($returnASCII);


        $jsonBody = json_decode($packDetails);
        $packData = isset($jsonBody->body->Pack) ? $jsonBody->body->Pack : '';

        return $packData;
    }

    function callGetPack($request)
    {

        if (!empty($request)) {

            $url = env('PACK_URL', true);
            $type = 'POST';
            $api_key = null;
            $api_passwd = null;

            return $this->executeCurl($url, $type, $request, $api_key, $api_passwd);
        } else {
            return null;
        }
    }

    function encode_items($array)
    {
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $array[$key] = $this->encode_items($value);
            } else {
                $array[$key] = mb_convert_encoding($value, 'ASCII');
            }
        }

        return $array;
    }

    function executeCurl($url, $type, $data, $api_key, $api_passwd)
    {
        $ch = curl_init();
        // curl_setopt($ch, CURLOPT_USERPWD, "$api_key:$api_passwd");
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/json"
            // "Authorization: Basic ".$authKey
        ));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE); // temporarily disable SSL verification
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE); // temporarily disable SSL verification

        if ($type == "POST") {
            curl_setopt($ch, CURLOPT_POST, TRUE);
            $json = json_encode(new ArrayValue($data));
            $payload = '"' . addslashes($json) . '"';

            $payload = json_encode($data);   // NEW Change **************************************
            curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        }

        if ($type == "PUT") {
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
            $json = json_encode(new ArrayValue($data));
            $payload = '"' . addslashes($json) . '"';
            curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        }

        $result = curl_exec($ch);


        if (curl_errno($ch)) {
            echo 'Request Error:' . curl_error($ch);
            return false;
        }

        return $result;
    }
}

class ArrayValue implements JsonSerializable
{
    public function __construct(array $array)
    {
        $this->array = $array;
    }

    public function jsonSerialize()
    {
        return $this->array;
    }
}
